@@ -443,7 +443,8 @@ dispose (GObject *object)
 		}
 	}
 
-	g_slist_free_full (priv->lines, g_free);
+	g_slist_foreach (priv->lines, (GFunc) g_free, NULL);
+	g_slist_free (priv->lines);
 
 done:
 	G_OBJECT_CLASS (applet_vpn_request_parent_class)->dispose (object);
