@@ -72,7 +72,9 @@ get_active_connection (GtkTreeView *treeview)
 	if (gtk_tree_model_get_iter (model, &iter, (GtkTreePath *) selected_rows->data))
 		gtk_tree_model_get (model, &iter, COL_CONNECTION, &connection, -1);
 
-	g_list_free_full (selected_rows, (GDestroyNotify) gtk_tree_path_free);
+	/* free memory */
+	g_list_foreach (selected_rows, (GFunc) gtk_tree_path_free, NULL);
+	g_list_free (selected_rows);
 
 	/* gtk_tree_model_get() will have reffed connection, but we don't
 	 * need that since we know the model will continue to hold a ref.
@@ -276,12 +278,8 @@ really_add_connection (FUNC_TAG_NEW_CONNECTION_RESULT_IMPL,
 		return;
 	}
 
-#if NM_LIBNM_COMPAT_PROXY_SUPPORTED
-	G_GNUC_BEGIN_IGNORE_DEPRECATIONS
 	if (connection_supports_proxy (connection) && !nm_connection_get_setting_proxy (connection))
 		nm_connection_add_setting (connection, nm_setting_proxy_new ());
-	G_GNUC_END_IGNORE_DEPRECATIONS
-#endif
 	if (connection_supports_ip4 (connection) && !nm_connection_get_setting_ip4_config (connection))
 		nm_connection_add_setting (connection, nm_setting_ip4_config_new ());
 	if (connection_supports_ip6 (connection) && !nm_connection_get_setting_ip6_config (connection))
