@@ -65,7 +65,7 @@ NMConnectionList *nm_connection_list_new (void);
 void              nm_connection_list_set_type (NMConnectionList *list, GType ctype);
 
 void              nm_connection_list_present (NMConnectionList *list);
-void              nm_connection_list_create (NMConnectionList *list, GType ctype, const char *detail, NMConnection *connection);
+void              nm_connection_list_create (NMConnectionList *list, GType ctype, const char *detail, const char *import_filename);
 void              nm_connection_list_edit (NMConnectionList *list, const gchar *uuid);
 void              nm_connection_list_add (NMConnectionList *list);
 
