@@ -110,7 +110,7 @@ ce_page_vpn_new (NMConnectionEditor *editor,
 
 	priv->plugin = vpn_get_plugin_by_service (service_type);
 	if (!priv->plugin) {
-		g_set_error (error, NMA_ERROR, NMA_ERROR_GENERIC, _("Could not find VPN plugin for '%s'."), service_type);
+		g_set_error (error, NMA_ERROR, NMA_ERROR_GENERIC, _("Could not find VPN plugin for “%s”."), service_type);
 		g_object_unref (self);
 		return NULL;
 	}
@@ -119,7 +119,7 @@ ce_page_vpn_new (NMConnectionEditor *editor,
 	priv->editor = nm_vpn_editor_plugin_get_editor (priv->plugin, CE_PAGE (self)->connection, &local);
 	if (!priv->editor) {
 		g_set_error (error, NMA_ERROR, NMA_ERROR_GENERIC,
-		             _("Could not load editor VPN plugin for '%s' (%s)."),
+		             _("Could not load editor VPN plugin for “%s” (%s)."),
 		             service_type, local ? local->message : _("unknown failure"));
 		g_clear_error (&local);
 		g_object_unref (self);
@@ -191,105 +191,55 @@ typedef struct {
 	gpointer user_data;
 } NewVpnInfo;
 
+typedef void (*VpnImportSuccessCallback) (NMConnection *connection, gpointer user_data);
+
+typedef struct {
+	VpnImportSuccessCallback callback;
+	gpointer user_data;
+} ActionInfo;
+
 static void
-import_cb (NMConnection *connection, gpointer user_data)
+complete_vpn_connection (NMConnection *connection, NMClient *client)
 {
-	NewVpnInfo *info = (NewVpnInfo *) user_data;
-	NMSettingConnection *s_con;
-	NMSettingVpn *s_vpn;
-	const char *service_type;
-	char *s;
-	GError *error = NULL;
-
-	/* Basic sanity checks of the connection */
-	s_con = nm_connection_get_setting_connection (connection);
-	if (!s_con) {
-		s_con = NM_SETTING_CONNECTION (nm_setting_connection_new ());
-		nm_connection_add_setting (connection, NM_SETTING (s_con));
-	}
-
-	s = (char *) nm_setting_connection_get_id (s_con);
-	if (!s) {
-		const GPtrArray *connections;
-
-		connections = nm_client_get_connections (info->client);
-		s = ce_page_get_next_available_name (connections, _("VPN connection %d"));
-		g_object_set (s_con, NM_SETTING_CONNECTION_ID, s, NULL);
-		g_free (s);
-	}
-
-	s = (char *) nm_setting_connection_get_connection_type (s_con);
-	if (!s || strcmp (s, NM_SETTING_VPN_SETTING_NAME))
-		g_object_set (s_con, NM_SETTING_CONNECTION_TYPE, NM_SETTING_VPN_SETTING_NAME, NULL);
-
-	s = (char *) nm_setting_connection_get_uuid (s_con);
-	if (!s) {
-		s = nm_utils_uuid_generate ();
-		g_object_set (s_con, NM_SETTING_CONNECTION_UUID, s, NULL);
-		g_free (s);
-	}
-
-	s_vpn = nm_connection_get_setting_vpn (connection);
-	service_type = s_vpn ? nm_setting_vpn_get_service_type (s_vpn) : NULL;
-
-	if (!service_type || !strlen (service_type)) {
-		g_object_unref (connection);
-		connection = NULL;
-
-		error = g_error_new_literal (NMA_ERROR, NMA_ERROR_GENERIC,
-		                             _("The VPN plugin failed to import the VPN connection correctly\n\nError: no VPN service type."));
-	}
-
-	info->result_func (connection, FALSE, error, info->user_data);
-	g_clear_error (&error);
-	g_object_unref (info->client);
-	g_slice_free (NewVpnInfo, info);
-}
-
-void
-vpn_connection_import (GtkWindow *parent,
-                       const char *detail,
-                       gpointer detail_data,
-                       NMClient *client,
-                       PageNewConnectionResultFunc result_func,
-                       gpointer user_data)
-{
-	NewVpnInfo *info;
-
-	info = g_slice_new (NewVpnInfo);
-	info->result_func = result_func;
-	info->client = g_object_ref (client);
-	info->user_data = user_data;
-	vpn_import (import_cb, info);
+	ce_page_complete_connection (connection,
+	                             _("VPN connection %d"),
+	                             NM_SETTING_VPN_SETTING_NAME,
+	                             FALSE,
+	                             client);
 }
 
 #define NEW_VPN_CONNECTION_PRIMARY_LABEL _("Choose a VPN Connection Type")
-#define NEW_VPN_CONNECTION_SECONDARY_LABEL _("Select the type of VPN you wish to use for the new connection.  If the type of VPN connection you wish to create does not appear in the list, you may not have the correct VPN plugin installed.")
+#define NEW_VPN_CONNECTION_SECONDARY_LABEL _("Select the type of VPN you wish to use for the new connection. If the type of VPN connection you wish to create does not appear in the list, you may not have the correct VPN plugin installed.")
 
 static gboolean
-vpn_type_filter_func (GType type, gpointer user_data)
+vpn_type_filter_func (FUNC_TAG_NEW_CONNECTION_TYPE_FILTER_IMPL,
+                      GType type,
+                      gpointer user_data)
 {
 	return type == NM_TYPE_SETTING_VPN;
 }
 
 static void
-vpn_type_result_func (NMConnection *connection, gpointer user_data)
+vpn_type_result_func (FUNC_TAG_NEW_CONNECTION_RESULT_IMPL,
+                      NMConnection *connection,
+                      gpointer user_data)
 {
 	NewVpnInfo *info = user_data;
 
-	info->result_func (connection, connection == NULL, NULL, info->user_data);
+	info->result_func (FUNC_TAG_PAGE_NEW_CONNECTION_RESULT_CALL, connection, connection == NULL, NULL, info->user_data);
 	g_slice_free (NewVpnInfo, info);
 }
 
 void
-vpn_connection_new (GtkWindow *parent,
+vpn_connection_new (FUNC_TAG_PAGE_NEW_CONNECTION_IMPL,
+                    GtkWindow *parent,
                     const char *detail,
                     gpointer detail_data,
+                    NMConnection *connection,
                     NMClient *client,
                     PageNewConnectionResultFunc result_func,
                     gpointer user_data)
 {
-	NMConnection *connection;
 	NMSetting *s_vpn;
 	const char *service_type;
 	gs_free char *service_type_free = NULL;
@@ -299,8 +249,9 @@ vpn_connection_new (GtkWindow *parent,
 	gssize split_idx, l;
 	const char *add_detail_key = NULL;
 	const char *add_detail_val = NULL;
+	gs_unref_object NMConnection *connection_tmp = NULL;
 
-	if (!detail) {
+	if (!detail && !connection) {
 		NewVpnInfo *info;
 
 		/* This will happen if nm-c-e is launched from the command line
@@ -318,63 +269,61 @@ vpn_connection_new (GtkWindow *parent,
 		return;
 	}
 
-	service_type = detail;
-	add_detail_key = vpn_data ? vpn_data->add_detail_key : NULL;
-	add_detail_val = vpn_data ? vpn_data->add_detail_val : NULL;
+	connection = _ensure_connection_other (connection, &connection_tmp);
+	if (detail) {
+		service_type = detail;
+		add_detail_key = vpn_data ? vpn_data->add_detail_key : NULL;
+		add_detail_val = vpn_data ? vpn_data->add_detail_val : NULL;
 
-	service_type_free = nm_vpn_plugin_info_list_find_service_type (vpn_get_plugin_infos (), detail);
-	if (service_type_free)
-		service_type = service_type_free;
-	else if (!vpn_data) {
-		/* when called without @vpn_data, it means that @detail may contain "<SERVICE_TYPE>:<ADD_DETAIL>".
-		 * Try to parse them by spliting @detail at the colons and try to interpret the first part as
-		 * @service_type and the remainder as add-detail. */
-		l = strlen (detail);
-		for (split_idx = 1; split_idx < l - 1; split_idx++) {
-			if (detail[split_idx] == ':') {
-				gs_free char *detail_main = g_strndup (detail, split_idx);
-				NMVpnEditorPlugin *plugin;
+		service_type_free = nm_vpn_plugin_info_list_find_service_type (vpn_get_plugin_infos (), detail);
+		if (service_type_free)
+			service_type = service_type_free;
+		else if (!vpn_data) {
+			/* when called without @vpn_data, it means that @detail may contain "<SERVICE_TYPE>:<ADD_DETAIL>".
+			 * Try to parse them by spliting @detail at the colons and try to interpret the first part as
+			 * @service_type and the remainder as add-detail. */
+			l = strlen (detail);
+			for (split_idx = 1; split_idx < l - 1; split_idx++) {
+				if (detail[split_idx] == ':') {
+					gs_free char *detail_main = g_strndup (detail, split_idx);
+					NMVpnEditorPlugin *plugin;
 
-				service_type_free = nm_vpn_plugin_info_list_find_service_type (vpn_get_plugin_infos (), detail_main);
-				if (!service_type_free)
-					continue;
-				plugin = vpn_get_plugin_by_service (service_type_free);
-				if (!plugin) {
-					g_clear_pointer (&service_type_free, g_free);
-					continue;
+					service_type_free = nm_vpn_plugin_info_list_find_service_type (vpn_get_plugin_infos (), detail_main);
+					if (!service_type_free)
+						continue;
+					plugin = vpn_get_plugin_by_service (service_type_free);
+					if (!plugin) {
+						g_clear_pointer (&service_type_free, g_free);
+						continue;
+					}
+
+					/* we found a @service_type. Try to use the remainder as add-detail. */
+					service_type = service_type_free;
+					if (nm_vpn_editor_plugin_get_service_add_detail (plugin, service_type, &detail[split_idx + 1],
+					                                                 NULL, NULL,
+					                                                 &add_detail_key_free, &add_detail_val_free, NULL)
+					    && add_detail_key_free && add_detail_key_free[0]
+					    && add_detail_val_free && add_detail_val_free[0]) {
+						add_detail_key = add_detail_key_free;
+						add_detail_val = add_detail_val_free;
+					}
+					break;
 				}
-
-				/* we found a @service_type. Try to use the remainder as add-detail. */
-				service_type = service_type_free;
-				if (nm_vpn_editor_plugin_get_service_add_detail (plugin, service_type, &detail[split_idx + 1],
-				                                                 NULL, NULL,
-				                                                 &add_detail_key_free, &add_detail_val_free, NULL)
-				    && add_detail_key_free && add_detail_key_free[0]
-				    && add_detail_val_free && add_detail_val_free[0]) {
-					add_detail_key = add_detail_key_free;
-					add_detail_val = add_detail_val_free;
-				}
-				break;
 			}
 		}
+		if (!service_type)
+			service_type = detail;
+
+		s_vpn = nm_setting_vpn_new ();
+		g_object_set (s_vpn, NM_SETTING_VPN_SERVICE_TYPE, service_type, NULL);
+
+		if (add_detail_key)
+			nm_setting_vpn_add_data_item ((NMSettingVpn *) s_vpn, add_detail_key, add_detail_val);
+
+		nm_connection_add_setting (connection, s_vpn);
 	}
-	if (!service_type)
-		service_type = detail;
 
-	connection = ce_page_new_connection (_("VPN connection %d"),
-	                                     NM_SETTING_VPN_SETTING_NAME,
-	                                     FALSE,
-	                                     client,
-	                                     user_data);
-	s_vpn = nm_setting_vpn_new ();
-	g_object_set (s_vpn, NM_SETTING_VPN_SERVICE_TYPE, service_type, NULL);
+	complete_vpn_connection (connection, client);
 
-	if (add_detail_key)
-		nm_setting_vpn_add_data_item ((NMSettingVpn *) s_vpn, add_detail_key, add_detail_val);
-
-	nm_connection_add_setting (connection, s_vpn);
-
-	(*result_func) (connection, FALSE, NULL, user_data);
+	(*result_func) (FUNC_TAG_PAGE_NEW_CONNECTION_RESULT_CALL, connection, FALSE, NULL, user_data);
 }
-
-
